/*
*   control:
*    000  add, signed (overflow)
*    001  add, unsigned (no overflow)
*    010  subtract, signed (overflow)
*    011  subtract, unsigned (no overflow)
* --------------------------------------------
*    100  AND
*    101  OR
*    110  XOR
*    111  NOR
*/  
module alu(a,b,control,c,zero,ovfl);
parameter BITS = 32;
input [BITS-1:0] a, b;
input [2:0] control;
output [BITS-1:0] c;
output zero, ovfl;

wire subtract;
assign subtract = control[1];

// adder/subtractor
wire [BITS-1:0] sum;
wire cin, cout;
assign cin = (subtract? 1'b1: 1'b0);
assign {cout, sum} = a+(subtract? ~b: b)+cin;

// logic unit
reg [BITS-1:0] out;
always
	case (control)
	2'b00: out = a&b;
	2'b01: out = a|b;
	2'b10: out = a^b;
	2'b11: out = ~(a|b);
	endcase

wire overflow;
assign zero = (adder_out==0);
assign overflow  = (adder_out[BITS-1]? (~a[BITS-1])&(~b[BITS-1]): a[BITS-1]&b[BITS-1] );
assign ovfl = (control[0]? 1'b0: overflow);

assign c = (control[2]? out: sum);

endmodule
